package com.diagbot.client;

import com.diagbot.client.bean.SearchData;
import com.diagbot.client.hystrix.AiptServiceHystrix;
import com.diagbot.dto.ConceptBaseDTO;
import com.diagbot.dto.ConceptRetrievalDTO;
import com.diagbot.dto.GetTopPatientInfoDTO;
import com.diagbot.dto.PushDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.vo.ConceptSearchVO;
import com.diagbot.vo.ConceptUsualVO;
import com.diagbot.vo.GetTopPatientInfoVO;
import com.diagbot.vo.RetrievalVO;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

/**
 * @Description:调用中间层服务
 * @Author: Mark Huang
 * @time: 2019/03/25 19:15
 */
@FeignClient(value = "aipt-service", fallback = AiptServiceHystrix.class)
public interface AiptServiceClient {
    /**
     * 推理
     *
     * @param searchData
     * @return
     */
    @PostMapping("/push/pushInner")
    RespDTO<PushDTO> pushInner(@RequestBody SearchData searchData);
    
    /**
     * 获取页面顶部病人医生科室信息
     *
     * @param getTopPatientInfoVO
     * @return
     */
    @PostMapping("/patientInfo/getTopPatientInfo")
    RespDTO<GetTopPatientInfoDTO> getTopPatientInfo(@RequestBody GetTopPatientInfoVO getTopPatientInfoVO);

    /**
     * 常见症状
     *
     * @param conceptUsualVO
     * @return
     */
    @PostMapping(value = "/concept/getConceptUsual")
    RespDTO<List<ConceptBaseDTO>> getConceptUsual(@RequestBody @Valid ConceptUsualVO conceptUsualVO);

    /**
     * 标签检索
     *
     * @param retrievalVO
     * @return
     */
    @PostMapping("/concept/retrivelConceptInfo")
    RespDTO<List<ConceptRetrievalDTO>> retrivelConceptInfo(@RequestBody @Valid RetrievalVO retrievalVO);

    /**
     * 根据概念Id列表获取概念列表Map
     *
     * @param conceptSearchVO 搜索参数
     * @return 术语id和术语 Map
     */
    @PostMapping("/concept/getConceptMap")
    RespDTO<Map<Long, String>> getConceptMap(@RequestBody ConceptSearchVO conceptSearchVO);
}