package com.diagbot.facade;

import java.util.Arrays;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

import org.assertj.core.util.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.EvaluatorCalDTO;
import com.diagbot.entity.InquiryEvaluator;
import com.diagbot.entity.InquiryInfo;
import com.diagbot.entity.PhysicalVal;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.PhysicalTypeEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.impl.InquiryEvaluatorServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.vo.AddEvaluatorVO;
import com.diagbot.vo.EvaluatorCalVO;

/**
 * @author: zhoutg
 * @time: 2018/8/6 9:11
 */
@Component
public class InquiryEvaluatorFacade extends InquiryEvaluatorServiceImpl {
	@Autowired
    private PhysicalInfoFacade physicalInfoFacade;
	@Autowired
    private InquiryInfoFacade inquiryInfoFacade;

	/**
	 * 儿童体质辨识--添加
	 * @param addDeptInfoVO
	 * @return
	 */
    public Boolean addEvaluator(AddEvaluatorVO addDeptInfoVO) {
        QueryWrapper<InquiryInfo> inquiryInfoQe = new QueryWrapper<>();
        inquiryInfoQe.eq("is_deleted", IsDeleteEnum.N.getKey());
        inquiryInfoQe.eq("hospital_id", addDeptInfoVO.getHospitalId());
        inquiryInfoQe.eq("hospital_dept_id", addDeptInfoVO.getHospitalDeptId());
        inquiryInfoQe.eq("doctor_id", addDeptInfoVO.getDoctorId());
        inquiryInfoQe.eq("patient_id", addDeptInfoVO.getPatientId());
        inquiryInfoQe.eq("inquiry_code", addDeptInfoVO.getInquiryCode());
        InquiryInfo inquiryInfo = inquiryInfoFacade.getOne(inquiryInfoQe);
        
        Date now = DateUtil.now();
        
        if(inquiryInfo==null){
        	inquiryInfo = new InquiryInfo();
        	inquiryInfo.setGmtCreate(now);
        }
        
        BeanUtil.copyProperties(addDeptInfoVO, inquiryInfo);
        inquiryInfo.setGmtModified(now);
        inquiryInfoFacade.saveOrUpdate(inquiryInfo);
        
        EvaluatorCalVO evaluatorCalVO = new EvaluatorCalVO();
        BeanUtil.copyProperties(addDeptInfoVO, evaluatorCalVO);
        EvaluatorCalDTO evaluatorCalDTO = evaluatorCalPrav(evaluatorCalVO);

        InquiryEvaluator inquiryEvaluator = new InquiryEvaluator();
        BeanUtil.copyProperties(evaluatorCalDTO, inquiryEvaluator);
        inquiryEvaluator.setInquiryId(inquiryInfo.getId());
        inquiryEvaluator.setGmtCreate(now);
        inquiryEvaluator.setGmtModified(now);
        
        if (!save(inquiryEvaluator)) {
            throw new CommonException(CommonErrorCode.INSERT_DATA_FAILED, "评估结果提交失败！");
        }
        
        return true;
    }
    
    /**
     * 儿童体质辨识--体质计算
     * @param evaluatorCalVO
     * @return
     */
    public EvaluatorCalDTO evaluatorCal(EvaluatorCalVO evaluatorCalVO){
    	EvaluatorCalDTO evaluatorCalDTO = evaluatorCalPrav(evaluatorCalVO);
        evaluatorCalDTO.setPhysicalInfoList(Lists.newArrayList(physicalInfoFacade.listByIds(Arrays.asList(evaluatorCalDTO.getPhysicalTypes().split(",")).stream().map(i->Integer.parseInt(i)).collect(Collectors.toList()))));
        return evaluatorCalDTO;
    }
    
    /**
     * 进行体质计算
     * @param evaluatorCalVO
     * @return
     */
    private EvaluatorCalDTO evaluatorCalPrav(EvaluatorCalVO evaluatorCalVO){
    	List<PhysicalVal> physicalValList = valueSort(evaluatorCalVO);
        Double max = physicalValList.get(0).getValue();
        Double min = physicalValList.get(4).getValue();
        
        String physicalRes = null,physicalTypes=null;
        if(max>=50){
            // 最大值有多个【是XX体质兼有XX体质】,最大值只有1个【是XX体质】
            if(physicalValList.get(1).getValue()>=50){
            	physicalRes = "是" 
            				+ PhysicalTypeEnum.getName(physicalValList.get(0).getEnumNo())
            				+ "兼有" 
            				+ PhysicalTypeEnum.getName(physicalValList.get(1).getEnumNo());
            	physicalTypes = physicalValList.get(0).getEnumNo() 
            				  + "," 
            				  + physicalValList.get(1).getEnumNo();
            }else{
            	physicalRes = "是" + PhysicalTypeEnum.getName(physicalValList.get(0).getEnumNo());
            	physicalTypes = String.valueOf(physicalValList.get(0).getEnumNo());
            }
        }else if(max<30){
            // 同时其他5种体质转化分均<30分, max[2/3/4/5/6] < 30
        	physicalRes = "平和体质";
        	physicalTypes = String.valueOf(PhysicalTypeEnum.mild.getKey());
        }else{
            if(evaluatorCalVO.getMildValue().doubleValue()>=60.0){
                if(max<50&&min>=30){
                    // 同时其他5种体质转化分均<50分,min[2/3/4/5/6] >= 30 and max[2/3/4/5/6] < 50
                	physicalRes = "基本是平和体质";
                	physicalTypes = String.valueOf(PhysicalTypeEnum.mild.getKey());
                }else if(max<50&&max>=30&&min<30){
                    // 同时其他5种体质转化分有30-49分,max[2/3/4/5/6] < 50 and max[2/3/4/5/6] >= 30 and min[2/3/4/5/6] < 30
                	physicalRes = "基本是平和体质兼倾向" + PhysicalTypeEnum.getName(physicalValList.get(0).getEnumNo());
                	physicalTypes = PhysicalTypeEnum.mild.getKey() + "," + physicalValList.get(0).getEnumNo();
                }
            }else{
                if(physicalValList.get(0).getValue().equals(physicalValList.get(1).getValue())){
                	physicalRes = "倾向是" 
                                + PhysicalTypeEnum.getName(physicalValList.get(0).getEnumNo())
                                + "兼有" 
                                + PhysicalTypeEnum.getName(physicalValList.get(1).getEnumNo());
                	physicalTypes = physicalValList.get(0).getEnumNo() + "," + physicalValList.get(1).getEnumNo();
                }else{
                	physicalRes = "倾向是" + PhysicalTypeEnum.getName(physicalValList.get(0).getEnumNo());
                	physicalTypes = String.valueOf(physicalValList.get(0).getEnumNo());
                }
            }
        }
    	
    	EvaluatorCalDTO evaluatorCalDTO = new EvaluatorCalDTO();
    	BeanUtil.copyProperties(evaluatorCalVO, evaluatorCalDTO);
    	evaluatorCalDTO.setPhysicalRes(physicalRes);
        evaluatorCalDTO.setPhysicalTypes(physicalTypes);
    	
    	return evaluatorCalDTO;
    }
    
    /**
     * 对偏颇体质排序
     * @param evaluatorCalVO
     * @return
     */
    private List<PhysicalVal> valueSort(EvaluatorCalVO evaluatorCalVO){
        List<PhysicalVal> res = ListUtil.newArrayList();
        //气虚体质转化分
        PhysicalVal faintValue = new PhysicalVal();
        faintValue.setValue(evaluatorCalVO.getFaintValue().doubleValue());
        faintValue.setEnumNo(PhysicalTypeEnum.faint.getKey());
        faintValue.setOrderNo(1);
        res.add(faintValue);
        //积滞体质转化分
        PhysicalVal stagnantValue = new PhysicalVal();
        stagnantValue.setValue(evaluatorCalVO.getStagnantValue().doubleValue());
        stagnantValue.setEnumNo(PhysicalTypeEnum.stagnant.getKey());
        stagnantValue.setOrderNo(2);
        res.add(stagnantValue);
        //湿滞体质转化分
        PhysicalVal wetValue = new PhysicalVal();
        wetValue.setValue(evaluatorCalVO.getWetValue().doubleValue());
        wetValue.setEnumNo(PhysicalTypeEnum.wet.getKey());
        wetValue.setOrderNo(3);
        res.add(wetValue);
        //心肝火旺体质转化分
        PhysicalVal heartValue = new PhysicalVal();
        heartValue.setValue(evaluatorCalVO.getHeartValue().doubleValue());
        heartValue.setEnumNo(PhysicalTypeEnum.heart.getKey());
        heartValue.setOrderNo(4);
        res.add(heartValue);
        //异禀体质转化分
        PhysicalVal exceptValue = new PhysicalVal();
        exceptValue.setValue(evaluatorCalVO.getExceptValue().doubleValue());
        exceptValue.setEnumNo(PhysicalTypeEnum.except.getKey());
        exceptValue.setOrderNo(5);
        res.add(exceptValue);

        return res.stream()
                .sorted(Comparator.comparing(PhysicalVal::getValue)
                .reversed()
                .thenComparing(PhysicalVal::getOrderNo))
                .collect(Collectors.toList());
    }
    
}
