package com.diagbot.vo;

import java.math.BigDecimal;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;

import org.hibernate.validator.constraints.Range;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;

/**
 * @Description:
 * @author: gaodm
 * @time: 2019/5/30 10:53
 */
@ApiModel(value="儿童体质辨识--添加--接口传参")
@Getter
@Setter
public class AddEvaluatorVO {
	
	/**
     * 医院id
     */
	@ApiModelProperty(value="医院id",required=true)
	@NotNull(message="医院id必传")
    private Long hospitalId;
	
    /**
     * 医院编码
     */
	@ApiModelProperty(value="医院编码",required=true)
	@NotBlank(message="医院编码必传")
    private String hospitalCode;

    /**
     * 科室id
     */
	@ApiModelProperty(value="科室id",required=true)
	@NotNull(message="科室id必传")
    private Long hospitalDeptId;

    /**
     * 医生id
     */
	@ApiModelProperty(value="医生id",required=true)
	@NotNull(message="医生id必传")
    private Long doctorId;

    /**
     * 患者id
     */
	@ApiModelProperty(value="患者id",required=true)
	@NotNull(message="患者id必传")
    private Long patientId;

    /**
     * 就诊序列号
     */
	@ApiModelProperty(value="就诊序列号",required=true)
	@NotBlank(message="就诊序列号必传")
    private String inquiryCode;
	
	/**
     * 就诊状态(0待接诊，1接诊中，2完成接诊)
     */
	@ApiModelProperty(value="就诊状态(0待接诊，1接诊中，2完成接诊)")
	@Range(min=0,max=2,message="就诊状态必须是0、1、2")
	private Integer regVisitedState=1;

    /**
     * 病历分类(1:门诊,2:住院)
     */
	@ApiModelProperty(value="病历分类(1:门诊,2:住院)")
	@Range(min=1,max=2,message="病历分类必须是1、2")
    private Integer type=1;
	
	/**
     * 平和体质转化分
     */
	@ApiModelProperty(value="平和体质转化分",required=true)
	@NotNull(message="平和体质转化分必传")
    private BigDecimal mildValue;

    /**
     * 气虚体质转化分
     */
    @ApiModelProperty(value="气虚体质转化分",required=true)
	@NotNull(message="气虚体质转化分必传")
    private BigDecimal faintValue;

    /**
     * 湿滞体质转化分
     */
    @ApiModelProperty(value="湿滞体质转化分",required=true)
	@NotNull(message="湿滞体质转化分必传")
    private BigDecimal wetValue;

    /**
     * 心肝火旺体质转化分
     */
    @ApiModelProperty(value="心肝火旺体质转化分",required=true)
	@NotNull(message="心肝火旺体质转化分必传")
    private BigDecimal heartValue;

    /**
     * 积滞体质转化分
     */
    @ApiModelProperty(value="积滞体质转化分",required=true)
	@NotNull(message="积滞体质转化分必传")
    private BigDecimal stagnantValue;

    /**
     * 异禀体质转化分
     */
    @ApiModelProperty(value="异禀体质转化分",required=true)
	@NotNull(message="异禀体质转化分必传")
    private BigDecimal exceptValue;
    
    /**
     * 体质结果
     */
	@ApiModelProperty(value="体质结果")
    private String physicalRes;

    /**
     * 需要显示的体质逗号隔开
     */
	@ApiModelProperty(value="需要显示的体质逗号隔开")
    private String physicalTypes;
	
}
